// Vikunja is a to-do list application to facilitate your life.
// Copyright 2018-2021 Vikunja and contributors. All rights reserved.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public Licensee as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public Licensee for more details.
//
// You should have received a copy of the GNU Affero General Public Licensee
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

package models

import (
	"code.vikunja.io/api/pkg/metrics"
	"encoding/json"

	"code.vikunja.io/api/pkg/events"
	"github.com/ThreeDotsLabs/watermill/message"
)

func RegisterListeners() {
	events.RegisterListener((&TaskCreatedEvent{}).TopicName(), &SendTaskCreatedNotification{})
	events.RegisterListener((&ListCreatedEvent{}).TopicName(), &IncreaseListCounter{})
	events.RegisterListener((&ListDeletedEvent{}).TopicName(), &DecreaseListCounter{})
}

type SendTaskCreatedNotification struct {
}

func (s *SendTaskCreatedNotification) Name() string {
	return "task.created.notification"
}

func (s *SendTaskCreatedNotification) Handle(payload message.Payload) (err error) {
	event := &TaskCreatedEvent{}
	err = json.Unmarshal(payload, event)
	if err != nil {
		return err
	}

	return nil
}

///////
// List Event Listeners

type IncreaseListCounter struct {
}

func (s *IncreaseListCounter) Name() string {
	return "increase.list.counter"
}

func (s *IncreaseListCounter) Handle(payload message.Payload) (err error) {
	metrics.UpdateCount(1, metrics.ListCountKey)
	return nil
}

type DecreaseListCounter struct {
}

func (s *DecreaseListCounter) Name() string {
	return "decrease.list.counter"
}

func (s *DecreaseListCounter) Handle(payload message.Payload) (err error) {
	metrics.UpdateCount(-1, metrics.ListCountKey)
	return nil
}
